//
//  ZplayFacebookAccountLoginAdapter.m
//  Zplay_Demo
//
//  Created by Liubin on 2017/3/10.
//  Copyright © 2017年 ZPLAY. All rights reserved.
//

#import "ZplayFacebookAccountLoginAdapter.h"
#import <FBSDKCoreKit/FBSDKCoreKit.h>
#import <FBSDKLoginKit/FBSDKLoginKit.h>
#import <FBSDKShareKit/FBSDKShareKit.h>
#import "SUCacheItem.h"
#import "SUCache.h"
#import "ZplayPlatformManager.h"

@interface ZplayFacebookAccountLoginAdapter ()<FBSDKSharingDelegate,FBSDKAppInviteDialogDelegate>
{
    FBSDKLoginManager* login;
    
    BOOL _isLogin;
    
    UIActivityIndicatorView *activity;

}
@property (copy, nonatomic) NSString *userHeadImageUrl;

@property (copy, nonatomic) NSString *userid;

@end

@implementation ZplayFacebookAccountLoginAdapter


+ (NSString *)loginType{
    return ZplayFacebookLoginAdapter;
}

+(void)load{
    [[ZplayLoginAdNetworkRegistry sharedRegistry]registerClass:self];
}

- (void)initializeLoginPlatforms{
    NSDictionary *plistDic = [[NSBundle mainBundle] infoDictionary];
    NSString *appid = [plistDic objectForKey:@"facebookid"];
    [FBSDKProfile enableUpdatesOnAccessTokenChange:YES];
    [FBSDKSettings setAppID:appid];
    
    login = [[FBSDKLoginManager alloc]init];
    
    //注册通知
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(_updateContent:)
                                                 name:FBSDKProfileDidChangeNotification
                                               object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(_accessTokenChanged:)
                                                 name:FBSDKAccessTokenDidChangeNotification
                                               object:nil];
}


/**
 登录
 
 @param message
 */
- (void)zplayPlatformLoginWithMessage:(id)message{
    _isLogin = YES;
    [self showActivity];
    [self zplayPlatformLogout];
    
    NSInteger slot = 0;
    FBSDKAccessToken *token = [SUCache itemForSlot:slot].token;
    if (token) {
        [self autoLoginWithToken:token isLogin:_isLogin];
    }
    else {
        [self newLogin];
    }
}




/**
 绑定

 @param message
 */
- (void)zplayPlatformBindWithMessage:(id)message{
    _isLogin = NO;
    [self showActivity];
    [self zplayPlatformLogout];
    [self newLogin];

}

/**
 邀请
 
 @param message
 */
- (void)zplayInviteUserFriendsWithMessage:(id)message{
    if (!message) {
        return;
    }
    NSString * appLinkUrl = [message objectForKey:@"fbAppLinkUrl"];
    FBSDKAppInviteContent *content = [[FBSDKAppInviteContent alloc] init];
    content.appLinkURL = [NSURL URLWithString:appLinkUrl];//[NSURL URLWithString:@"https://fb.me/319673994858989"];
//    content.appInvitePreviewImageURL = [NSURL URLWithString:@"http://hutui-img.zplay.cn/share/foreign/R1490692822M.jpeg"];
    [FBSDKAppInviteDialog showFromViewController:[self viewControllerLoginModalView]
                                     withContent:content
                                        delegate:self];
    
}

- (void)zplayPlatformLogout{
    [login logOut];
    NSInteger slot = 0;
    [SUCache deleteItemInSlot:slot];
    [FBSDKAccessToken setCurrentAccessToken:nil];
}


- (void)zplayLoginHandleOpenUrl:(id)platformMessage{
    
    if (platformMessage==nil) {
        return;
    }

    [[FBSDKApplicationDelegate sharedInstance] application:nil
                                                   openURL:[platformMessage objectForKey:@"url"]
                                                   options:[platformMessage objectForKey:@"options"]];
    
    [[FBSDKApplicationDelegate sharedInstance] application:nil
                                                   openURL:[platformMessage objectForKey:@"url"]
                                         sourceApplication:nil
                                                annotation:nil];
}

//再次登录
- (void)autoLoginWithToken:(FBSDKAccessToken *)token isLogin:(BOOL)isLogin{
    [FBSDKAccessToken setCurrentAccessToken:token];
    FBSDKGraphRequest *request = [[FBSDKGraphRequest alloc] initWithGraphPath:@"me" parameters:nil];
    [request startWithCompletionHandler:^(FBSDKGraphRequestConnection *connection, id result, NSError *error) {
        //token过期，删除存储的token和profile
        if (error) {
            NSLog(@"The user token is no longer valid.");
            NSInteger slot = 0;
            [SUCache deleteItemInSlot:slot];
            [FBSDKAccessToken setCurrentAccessToken:nil];
            [self loginOrBindingFailed];
        }
        //做登录完成的操作
        else {
            [self loginSuccessGetUserMessageWithToken:token isLogin:isLogin];
        }
    }];
}

//第一次登录
- (void)newLogin{
    login.loginBehavior = FBSDKLoginBehaviorNative;
    [login logInWithReadPermissions: @[@"public_profile",@"user_likes",@"user_friends"]
                 fromViewController:[self viewControllerLoginModalView]
                            handler:^(FBSDKLoginManagerLoginResult *result, NSError *error) {
                                
                                [self stopActivity];
                                if (error || result.isCancelled) {
                                    [self loginOrBindingFailed];
                                    //                                    NSLog(@"Unexpected login error: %@,%d", error,result.isCancelled);
                                    
                                }else{
                                    //                                    [self loginSuccessGetUserMessageWithToken:result.token isLogin:isLogin];
                                    
                                }
                            }];
}


    

//登录完成后获取用户信息
- (void)loginSuccessGetUserMessageWithToken:(FBSDKAccessToken *)token isLogin:(BOOL)isLogin{
    NSDictionary*params=[NSDictionary new];
    FBSDKGraphRequest *request = [[FBSDKGraphRequest alloc]
                                  initWithGraphPath:token.userID
                                  parameters:params
                                  HTTPMethod:@"GET"];
    
    //    __weak NSString *_facebookTokenString = result.token.tokenString;
    [request startWithCompletionHandler:^(FBSDKGraphRequestConnection *connection,
                                          id result,
                  
                        NSError *error) {
        NSLog(@"Facebook用户信息详情：%@",result);
                NSString*name=[result objectForKey:@"name"];
        self.userid=[result objectForKey:@"id"];
        
        if (!name || [name isEqualToString:@""] || !self.userid || [self.userid isEqualToString:@""] || !token.tokenString || [token.tokenString isEqualToString:@""]) {
            [self loginOrBindingFailed];
            return ;
        }
        
        self.userHeadImageUrl = [[NSString stringWithFormat:@"https://graph.facebook.com/%@/picture?type=normal&width=100&height=100",self.userid] stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
        
        NSString *userHead = (NSString *)
        CFBridgingRelease(CFURLCreateStringByAddingPercentEscapes(kCFAllocatorDefault,
                                                                  (CFStringRef)self.userHeadImageUrl,
                                                                  NULL,
                                                                  (CFStringRef)@"!*'();:@&=+$,/?%#[]",
                                                                  kCFStringEncodingUTF8));
        
        
        NSMutableDictionary *dict = [[NSMutableDictionary alloc]init];
        [dict setObject:self.userid==nil?@"":self.userid forKey:@"openid"];
        [dict setObject:token.tokenString==nil?@"":token.tokenString forKey:@"access_token"];
        [dict setObject:name==nil?@"":name forKey:@"nickname"];
        [dict setObject:userHead==nil?@"":userHead forKey:@"userHeadImage"];
        [dict setObject:[NSString stringWithFormat:@"%lu",(unsigned long)ZplayFacebookPlatform] forKey:@"platformType"];
        
        [self stopActivity];
        //登录
        if (_isLogin) {
            [self platfomrLoginSuccess:dict];
        }
        //绑定
        else{
            [self platformBindSucces:dict];
        }
    }];
    
    NSMutableDictionary *dict = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                                 @"picture,name",@"fields",
                                 token, @"access_token",
                                 nil];
    NSString *str = [NSString stringWithFormat:@"/me/friends"];
    FBSDKGraphRequest *friendRequest = [[FBSDKGraphRequest alloc]
                                        initWithGraphPath:str
                                        parameters:dict
                                        HTTPMethod:@"GET"];
    [friendRequest startWithCompletionHandler:^(FBSDKGraphRequestConnection *connection,
                                                id result,
                                                NSError *error) {
        if (result==nil) {
            [self loginOrBindingFailed];
            return ;
        }
        // Handle the result
        [self getFacebookUserFriendsSuccess:result];
    }];
}

#pragma mark - FacebookDelegate
- (void)_accessTokenChanged:(NSNotification *)notification
{
    FBSDKAccessToken *token = notification.userInfo[FBSDKAccessTokenChangeNewKey];
    if (!token) {
        [FBSDKAccessToken setCurrentAccessToken:nil];
    } else {
        NSInteger slot = 0;
        SUCacheItem *item = [SUCache itemForSlot:slot] ?: [[SUCacheItem alloc] init];
        if (![item.token isEqualToAccessToken:token]) {
            item.token = token;
            [SUCache saveItem:item slot:slot];
            if (activity) {
                [activity startAnimating];
            }
            [self loginSuccessGetUserMessageWithToken:item.token isLogin:_isLogin];
        }
    }
}

- (void)_updateContent:(NSNotification *)notification {
    
}


- (void)appInviteDialog:(FBSDKAppInviteDialog *)appInviteDialog didCompleteWithResults:(NSDictionary *)results{
    NSLog(@"邀请结果：%@",results);

    if (!results || [results isKindOfClass:[NSNull class]]) {
        [self platformFailed:ZplayPlatformInviteFriendsFailed];
    }
    else{
        if ([[results objectForKey:@"completionGesture"] isEqualToString:@"cancel"]) {
            [self platformFailed:ZplayPlatformInviteFriendsFailed];
        }
        else{
            [self inviteFacebookUserFriendsSuccess:results];
        }
    }
}

- (void)appInviteDialog:(FBSDKAppInviteDialog *)appInviteDialog didFailWithError:(NSError *)error{
    NSLog(@"邀请失败：%@",error);
    [self platformFailed:ZplayPlatformInviteFriendsFailed];
}



- (void)loginOrBindingFailed{
    if (_isLogin) {
        [self platformFailed:ZplayPlatformLoginFailed];
    }
    else{
        [self platformFailed:ZplayPlatformConversionFailed];
        
    }
}

- (void)platformFailed:(ZplayPlatformFailed)errcode{
    [self platfomrFailed:errcode];
    [activity stopAnimating]; // 开始旋转
}


- (void)showActivity{
    if (!activity) {
        activity = [[UIActivityIndicatorView alloc]initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleWhiteLarge];
    }
    activity.center = [self viewControllerLoginModalView].view.center;//只能设置中心，不能设置大小
    [[self viewControllerLoginModalView].view addSubview:activity];
    [activity startAnimating]; // 开始旋转
}

- (void)stopActivity{
    [activity stopAnimating]; // 开始旋转

}





@end
